// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <aoc/common/Grid.hpp>
#include <aoc/framework/LinesSolver.hpp>

class RaceCondition
    : public LinesSolver<int64_t, int64_t>
{
public:
    RaceCondition(const int threshold = 100);
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void finish() override;
private:
    static constexpr char getStartChar();
    static constexpr char getWallChar();
    static constexpr int getCheatLength();
    const std::array<Point2, 8> doubleSteps_{ Point2::down * 2, Point2::downRight, Point2::right * 2,
        Point2::upRight, Point2::up * 2, Point2::upLeft, Point2::left * 2, Point2::downLeft };
    int threshold_;
    void checkCheat(const Point2& position, Grid<int>& times);
};
